      program write_dkadT
      implicit none

      include './max.inc'
      include './param.inc'
      integer m,Nb,Nq,i,band,quad
      double precision dkadT(1:Nbmx,1:Nqmx,1:Nmax)

c     This program does nothing. It's purpose is to provide the format used for
c     the creation of "dkadT.txt" files that can be used by KARINE for computing
c     sensitivities of NERs to temperatures.
c     
c     Basically, you get:
c     d(NER)/d(T)=d(NER)/d(ka)*d(ka)/d(T)
c     
c     where T is the mean temperature in a given atmospheric layer.
c     In other words, KARINE knows how to compute d(NER)/d(ka); however,
c     you need to provide your (specific to each atmospheric configuration)
c     sensitivities of absorption coefficients to mean temperature in each atmospheric
c     layer, d(ka)/d(T).
c

c     First, compute dkadT coefficients as you wish; you need to compute the array
c     dkadT(band,quad,i) for:
c     band=1,Nb (index of spectral interval)
c     quad=1,Nq (index of quadrature)
c     i=1,m (index of atmospheric layer).
c
c     dkadT(band,quad,i) is the sensitivity of the absorption coefficient in spectral
c     interval index 'band', for quadrature element index 'quad', in atmospheric
c     layer index 'i', to the mean temperature (or temperature at the center) of
c     atmospheric layer index 'i': d[ ka(band,quad,i) ] / d[ T(i) ]
c     
c     It must be computed in inverse meters per kelvin.
c

c     Here your own code...
c     A simple example:
      m=5 ! number of gas layers
      Nb=2 ! number of spectral intervals
      Nq=3 !  quadrature order

      do i=1,m
         do band=1,Nb
            do quad=1,Nq
               dkadT(band,quad,i)=1.0D+0 ! m^(-1)/K
            enddo ! quad
         enddo ! band
      enddo ! i

c     Next, create the "dkadT.txt" file using the following format:
      open(12,file='./dkadT.txt')
      write(12,*) m,Nb,Nq
      do i=1,m
         do band=1,Nb
            write(12,'(100e17.9)') (dkadT(band,quad,i),quad=1,Nq)
         enddo                  ! band
      enddo ! i
      write(*,*) 'File ./dkadT.txt has been succesfully generated'

      end
