      program write_dkadT
      implicit none

      include './max.inc'
      include './param.inc'
      integer m,Nb,Nq,band,quad,i
      integer nspecies,mod,i_min,i_max
      double precision dkadx(1:Nbmx,1:Nqmx,1:Nmax,1:Nsmx)

c     This program does nothing. It's purpose is to provide the format used for
c     the creation of "dkadx.txt" files that can be used by KARINE for computing
c     sensitivities of NERs to concentrations.
c     
c     Basically, you get:
c     d(NER)/d(x)=d(NER)/d(ka)*d(ka)/d(x)
c     
c     where x is the mean concentration of a given species in a given atmospheric layer.
c     In the rest of this file, [x] refers to the unit used for your concentration
c     (number of particules per cubic meter, moles, kilograms per cubic meter, etc...)
c
c     In other words, KARINE knows how to compute d(NER)/d(ka); however,
c     you need to provide your (specific to each atmospheric configuration)
c     sensitivities of absorption coefficients to mean concentration in each atmospheric
c     layer, d(ka)/d(x).
c

c     First, compute dkadx coefficients as you wish; you need to compute the array
c     dkadx(band,quad,i) for:
c     band=1,Nb (index of spectral interval)
c     quad=1,Nq (quadrature index)
c     i=1,m (atmospheric layer index).
c
c     dkadx(band,quad,i) is the sensitivity of the absorption coefficient in spectral
c     interval index 'band', for quadrature point 'quad', in atmospheric layer index 'i',
c     to the mean concentration of a given species in atmospheric layer index 'i':
c     d[ ka(band,quad,i) ] / d[ x(i) ]
c     
c     It must be computed in inverse meters per [x].
c
c     Note: quadrature weights are provided within the "atmos_data.txt" input file.

c     Here your own code...
c     Example:
      m=5 ! number of gas layers
      Nb=2 ! number of spectral narrowbands
      Nq=3 ! quadrature order
      nspecies=3 ! number of chemical species dkadx is provided for
      i_min=1 ! index of the first atmospheric layer dkadx is provided for
      i_max=5 ! index of the last atmospheric layer dkadx is provided for

      do band=1,Nb
         do quad=1,Nq
            do i=i_min,i_max
               do mod=1,nspecies
                  dkadx(band,quad,i,mod)=1.0D+0 ! m^(-1)/[x]
               enddo            ! mod
            enddo               ! i
         enddo                  ! quad
      enddo                     ! band

c     Next, create the "dkadx.txt" file using the following format:
      open(12,file='./dkadx.txt')
      write(12,*) m,Nb,Nq
      write(12,*) "d(ka)/d(x) file for cloud particules"
      write(12,*) "Number of species (particules modes):"
      write(12,*) nspecies
      write(12,*) "Minimum layer d(ka)/d(x) is given in:"
      write(12,*) i_min
      write(12,*) "Maximum layer d(ka)/d(x) is given in:"
      write(12,*) i_max
      write(12,*) "Dimension used for d(ka)/d(x): m^(-1)/(#/cm^3)"
      write(12,*)
      do mod=1,nspecies
         write(12,*) 'd(ka)/d(x) for species number',mod
         do i=i_min,i_max
            do band=1,Nb
               write(12,'(200e17.9)') (dkadx(band,quad,i,mod),
     &              quad=1,Nq)
            enddo               ! band
         enddo                  ! i
         write(12,*)
      enddo                     ! mod
      close(12)
      write(*,*) 'File ./dkadx.txt has been succesfully generated'

      end
