c     Copyright (C)
c     2000 -2013 Universite Paul Sabatier,
c     2000 -2013 Centre National de la Recherche Scientifique
c     2000 -2013 Ecole des Mines d'Albi
c     2014-2018 Msostar
c
c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c     
c     sensitivities...
      subroutine analytic_dNERdT(band,quad)
      implicit none
      
      include 'max.inc'
      include 'data.inc'
      include 'options.inc'
      include 'param.inc'
      include 'dkadTp.inc'
      include 'dner_dT.inc'

      integer i,j,p,band,quad,md,it,jt,out
      double precision z1,z2,z3,z4,kai,kaj
      double precision B1,B2,B3,B4,tau_B
      double precision B_sol,B_esp
      double precision Planck,expint,lambda
      double precision tau_atmos
      double precision dBdT
      double precision dBdTg,dBdTs
c     dner_dT(i,j,p) est la drive de la PNE(i,j) p.r.  la temprature T(p)
      double precision dnerd_dT(0:2*(Nmax+1),0:2*(Nmax+1),0:Nmax+1)
      double precision Tz1,Tz2,Tz3,Tz4
      double precision dB0_dTp,dBesp_dTp
      double precision dB1_dTp,dB2_dTp
      double precision dB3_dTp,dB4_dTp
      double precision dtauB_dTp,dtaui_dTp,dtauj_dTp
      double precision taui,tauj
      double precision alpha,beta
      double precision tau_B1,tau_B2
      double precision dtauB1_dTp,dtauB2_dTp

      double precision func_dB0_dTp
      double precision func_dBesp_dTp
      double precision func_dB1_dTp
      double precision func_dB2_dTp
      double precision func_dtaui_dTp
      double precision func_dtauB_dTp

      double precision dnerdij_dTp_direct
      double precision dnerdmij_dTp_direct
      double precision dnerdij_dTp_indirect
      double precision tau_inf,dtau_inf_dTp
      double precision B1_ind,B2_ind,B3_ind,B4_ind
      double precision dB1_dTp_ind,dB2_dTp_ind
      double precision dB3_dTp_ind,dB4_dTp_ind
      double precision taui_ind,tauj_ind,tau_B_ind
      double precision dtaui_dTp_ind,dtauj_dTp_ind
      double precision dtauB_dTp_ind
      double precision alpha_ind,beta_ind

      double precision dNERij_dTp_dir,dNERij_dTp_ref
      double precision dNERis_dTp_dir,dNERis_dTp_ref
      double precision tauH,tau_below
      double precision dtauH_dTp,dtau_below_dTp

c     Computation of Planck intensities at ground and space levels
      lambda=(lambda_lo(band)+lambda_hi(band))/2.0D+0
      if (udif.eq.1) then
         B_sol=Planck(Tsol,lambda_lo(band),lambda_hi(band))
         B_esp=Planck(Tesp,lambda_lo(band),lambda_hi(band))
      else
         B_sol=udif_sol(band)
         B_esp=udif_esp(band)
      endif
      md=2*(m-1)

      do p=0,m+1
         dB0_dTp=func_dB0_dTp(p,lambda)
         dBesp_dTp=func_dBesp_dTp(p,lambda)
c     d(NER)/d(T) computation
         do i=1,md
            it=i/2+1
            z1=alti(i-1)
            z2=alti(i)
            call find_B(band,quad,z1,B1)
            call find_B(band,quad,z2,B2)
            dB1_dTp=func_dB1_dTp(i,p,lambda)
            dB2_dTp=func_dB2_dTp(i,p,lambda)
            call find_ka(band,quad,it,(z2+z1)/2.0D+0,kai)
            taui=kai*(z2-z1)
            dtaui_dTp=func_dtaui_dTp(i,p,dkadTp,band,quad)
c     Computation of NER(ground-layer i)
            call calc_tau_abs_mlay(band,quad,1,i-1,1,tau_B)
            tau_inf=tau_B
            dtauB_dTp=func_dtauB_dTp(0,i,p,dkadTp,band,quad)
            dtau_inf_dTp=dtauB_dTp
            
c     dner(0,i)/dT(p)
            if ((p.ge.0).and.(p.le.it+1)) then
               dnerd_dT(0,i,p)=2.0D+0*pi*em_sol(band)*(
     &              (dB1_dTp-dB0_dTp)*expint(3,tau_B)
     &              -(B1-B_sol)*dtauB_dTp*expint(2,tau_B)
     &              -(dB2_dTp-dB0_dTp)*expint(3,tau_B+taui)
     &              +(B2-B_sol)*(dtauB_dTp+dtaui_dTp)*
     &              expint(2,tau_B+taui)
     &              +1.0D+0/taui**2*(
     &              taui*(
     &              (dB2_dTp-dB1_dTp)*
     &              (expint(4,tau_B)-expint(4,tau_B+taui))
     &              -(B2-B1)*
     &              (dtauB_dTp*expint(3,tau_B)
     &              -(dtauB_dTp+dtaui_dTp)*
     &              expint(3,tau_B+taui)))
     &              -dtaui_dTp*(B2-B1)*
     &              (expint(4,tau_B)-expint(4,tau_B+taui))))
            else if ((p.ge.it+2).and.(p.le.m+1)) then
               dnerd_dT(0,i,p)=0.0D+0
            endif
            
c     Computation of NER(space-layer i)
            call calc_tau_abs_mlay(band,quad,i+1,md,1,tau_B)
            dtauB_dTp=func_dtauB_dTp(i,md+1,p,dkadTp,band,quad)
            
c     dner(i,m+1)/dT(p), p=2,m
            if ((p.ge.0).and.(p.le.it-2)) then
               dNERis_dTp_dir=0.0D+0
            else if ((p.ge.it-1).and.(p.le.m+1)) then
               dNERis_dTp_dir=2.0D+0*pi*(
     &              (dBesp_dTp-dB2_dTp)*expint(3,tau_B)
     &              -(B_esp-B2)*dtauB_dTp*expint(2,tau_B)
     &              -(dBesp_dTp-dB1_dTp)*expint(3,tau_B+taui)
     &              +(B_esp-B1)*(dtauB_dTp+dtaui_dTp)*
     &              expint(2,tau_B+taui)
     &              +1.0D+0/taui**2*(
     &              taui*((dB2_dTp-dB1_dTp)*(
     &              expint(4,tau_B)-expint(4,tau_B+taui))-
     &              (B2-B1)*(dtauB_dTp*expint(3,tau_B)-
     &              (dtauB_dTp+dtaui_dTp)*
     &              expint(3,tau_B+taui)))
     &              -dtaui_dTp*(B2-B1)*(
     &              expint(4,tau_B)-expint(4,tau_B+taui)))) ! W/m/m/K
            endif
            call calc_tau_abs_mlay(band,quad,1,md,1,tauH)
            dtauH_dTp=func_dtauB_dTp(0,md+1,p,dkadTp,band,quad)
            call calc_tau_abs_mlay(band,quad,1,i-1,1,tau_below)
            dtau_below_dTp=func_dtauB_dTp(0,i,p,dkadTp,band,quad)

            if (ref.eq.1) then ! specular reflexion
               dNERis_dTp_ref=2.0D+0*pi*(1.0D+0-em_sol(band))*(
     &              (dBesp_dTp-dB1_dTp)*expint(3,tau_below+tauH)
     &              -(B_esp-B1)*(dtau_below_dTp+dtauH_dTp)
     &              *expint(2,tau_below+tauH)
     &              -(dBesp_dTp-dB2_dTp)*expint(3,tau_below+tauH+taui)
     &              +(B_esp-B2)*(dtau_below_dTp+dtauH_dTp+dtaui_dTp)
     &              *expint(2,tau_below+tauH+taui)
     &              -1.0D+0/taui**2*(
     &              taui*((dB2_dTp-dB1_dTp)
     &              *(expint(4,tau_below+tauH)
     &              -expint(4,tau_below+tauH+taui))
     &              -(B2-B1)*(
     &              (dtau_below_dTp+dtauH_dTp)
     &              *expint(3,tau_below+tauH)
     &              -(dtau_below_dTp+dtauH_dTp+dtaui_dTp)
     &              *expint(3,tau_below+tauH+taui))
     &              )
     &              -dtaui_dTp*(B2-B1)*
     &              (expint(4,tau_below+tauH)
     &              -expint(4,tau_below+tauH+taui))
     &              )
     &              ) ! W/m/m/K
            else if (ref.eq.2) then ! diffuse reflexion
               dNERis_dTp_ref=4.0D+0*pi*(1.0D+0-em_sol(band))*(
     &              ((dBesp_dTp-dB1_dTp)*expint(3,tau_below)
     &              -(B_esp-B1)*dtau_below_dTp*expint(2,tau_below)
     &              -(dBesp_dTp-dB2_dTp)*expint(3,tau_below+taui)
     &              +(B_esp-B2)*(dtau_below_dTp+dtaui_dTp)
     &              *expint(2,tau_below+taui)
     &              -1.0D+0/taui**2*(
     &              taui*((dB2_dTp-dB1_dTp)*(expint(4,tau_below)
     &              -expint(4,tau_below+taui))
     &              -(B2-B1)*(dtau_below_dTp*expint(3,tau_below)
     &              -(dtau_below_dTp+dtaui_dTp)
     &              *expint(3,tau_below+taui))
     &              )-dtaui_dTp*(B2-B1)
     &              *(expint(4,tau_below)-expint(4,tau_below+taui))
     &              ))*expint(3,tauH)
     &              -((B_esp-B1)*expint(3,tau_below)
     &              -(B_esp-B2)*expint(3,tau_below+taui)
     &              -(B2-B1)/taui*(expint(4,tau_below)-
     &              expint(4,tau_below+taui))
     &              )*dtauH_dTp*expint(2,tauH)
     &              ) ! W/m/m/K
            endif

            dnerd_dT(i,md+1,p)=dNERis_dTp_dir+dNERis_dTp_ref ! W/m/m/K
            
c     Computation of NER(i,j)
            do j=i+1,md
               jt=j/2+1
               z3=alti(j-1)
               z4=alti(j)
               call find_B(band,quad,z3,B3)
               call find_B(band,quad,z4,B4)
               dB3_dTp=func_dB1_dTp(j,p,lambda)
               dB4_dTp=func_dB2_dTp(j,p,lambda)
               call find_ka(band,quad,jt,(z4+z3)/2.0D+0,kaj)
               tauj=kaj*(z4-z3)
               dtauj_dTp=func_dtaui_dTp(j,p,dkadTp,band,quad)
               call calc_tau_abs_mlay(band,quad,i+1,j-1,1,tau_B)
               dtauB_dTp=func_dtauB_dTp(i,j,p,dkadTp,band,quad)
               
c     Direct d(NERd(i,j))/d(Tp)
               if ((p.ge.1).and.(p.le.it-2)) then
                  dNERij_dTp_dir=0.0D+0
               else if ((p.ge.it-1).and.(p.le.jt+1)) then
                  call dNERij_dTp_direct(B1,B2,B3,B4,
     &                 dB1_dTp,dB2_dTp,dB3_dTp,dB4_dTp,
     &                 tau_B,taui,tauj,
     &                 dtaui_dTp,dtauj_dTp,dtauB_dTp,
     &                 dNERij_dTp_dir) ! dNERij_dTp_dir=W/m/m/K
               else if ((p.ge.jt+2).and.(p.le.m)) then
                  dNERij_dTp_dir=0.0D+0
               endif            ! p
               
c     Indirect d(NERd(i,j))/d(Tp)
               dNERij_dTp_ref=0.0D+0
               if (p.le.jt+1) then
                  if (ref.eq.1) then ! specular reflection at ground level
c     Both "dNERij_dTp_specular_reflection_virtual" and "dNERij_dTp_specular_reflection_real"
c     can be used for the computation of 'dNERij_dTp_ref'. They take the same
c     arguments, and the result will be the same. See comments in those two routines.

c                     call dNERij_dTp_specular_reflection_virtual(
c     &                    em_sol(band),
c     &                    B1,B2,B3,B4,
c     &                    dB1_dTp,dB2_dTp,dB3_dTp,dB4_dTp,
c     &                    tau_inf,tau_B,taui,tauj,
c     &                    dtaui_dTp,dtauj_dTp,
c     &                    dtau_inf_dTp,dtauB_dTp,
c     &                    dNERij_dTp_ref) ! dNERij_dTp_ref=W/m/m/K
                     call dNERij_dTp_specular_reflection_real(
     &                    em_sol(band),
     &                    B1,B2,B3,B4,
     &                    dB1_dTp,dB2_dTp,dB3_dTp,dB4_dTp,
     &                    tau_inf,tau_B,taui,tauj,
     &                    dtaui_dTp,dtauj_dTp,
     &                    dtau_inf_dTp,dtauB_dTp,
     &                    dNERij_dTp_ref) ! dNERij_dTp_ref=W/m/m/K

                  else if (ref.eq.2) then ! diffuse reflection at ground level
                     call dNERij_dTp_diffuse_reflection(em_sol(band),
     &                    B1,B2,B3,B4,
     &                    dB1_dTp,dB2_dTp,dB3_dTp,dB4_dTp,
     &                    tau_inf,tau_B,taui,tauj,
     &                    dtaui_dTp,dtauj_dTp,
     &                    dtau_inf_dTp,dtauB_dTp,
     &                    dNERij_dTp_ref)! dNERij_dTp_ref=W/m/m/K

                  endif ! over ref

               else if ((p.ge.jt+2).and.(p.le.m)) then
                  dNERij_dTp_ref=0.0D+0
               endif            ! p
               
c     Sum of both direct and indirect
c               dnerd_dT(i,j,p)=dnerdij_dTp_direct+dnerdij_dTp_indirect
c     using the new routine
               dnerd_dT(i,j,p)=dNERij_dTp_dir+dNERij_dTp_ref

c     Debug
c               if ((p.eq.1).and.(i.eq.1).and.(j.eq.2)) then
c                  write(*,*) 'dnerdij_dTp_direct=',dnerdij_dTp_direct
c                  write(*,*) 'dnerdij_dTp_indirect=',
c     &                 dnerdij_dTp_indirect
c                  write(*,*) 'dnerdij_dTp=',dnerd_dT(i,j,p)
c                  stop
c               endif
c     Debug
            enddo               ! over j
         enddo                  ! over i

c     Computation of NER(ground-space)
         call calc_tau_abs_mlay(band,quad,1,md,1,tau_atmos)
c     Sensitivities of ground-space NER to ground and space temperatures
         if (p.eq.0) then
            dnerd_dT(0,md+1,0)=-2.0D+0*pi*dB0_dTp*expint(3,tau_atmos) ! W/m2/m/K
         else if ((p.ge.1).and.(p.le.m)) then
            dnerd_dT(0,md+1,p)=2.0D+0*pi*(B_sol-B_esp)*(e(p)-e(p-1))
     &           *dkadTp(band,quad,p)
     &           *expint(2,tau_atmos) ! W/m2/m/K
         else if (p.eq.m+1) then
            dnerd_dT(0,md+1,m+1)=2.0D+0*pi*dBesp_dTp*expint(3,tau_atmos) ! W/m2/m/K
         endif

c     dNERd_dT from grid "alti" combination in order to get dNER_dT for grid "e"

c     ner(ground-i)
         dner_dT(0,1,p)=dnerd_dT(0,1,p)
         do i=2,m-1
            dner_dT(0,i,p)=dnerd_dT(0,2*(i-1),p)+dnerd_dT(0,2*i-1,p)
         enddo
         dner_dT(0,m,p)=dnerd_dT(0,md,p)

c     ner(space-i)
         dner_dT(1,m+1,p)=dnerd_dT(1,md+1,p)
         do i=2,m-1
            dner_dT(i,m+1,p)=dnerd_dT(2*(i-1),md+1,p)+
     &           dnerd_dT(2*i-1,md+1,p)
         enddo
         dner_dT(m,m+1,p)=dnerd_dT(md,md+1,p)

c     ner(i,j)
         do j=2,m-1
            dner_dT(1,j,p)=dnerd_dT(1,2*(j-1),p)+dnerd_dT(1,2*j-1,p)
         enddo
         dner_dT(1,m,p)=dnerd_dT(1,md,p)
         do i=2,m-1
            do j=i+1,m-1
               dner_dT(i,j,p)=dnerd_dT(2*(i-1),2*(j-1),p)+
     &              dnerd_dT(2*(i-1),2*j-1,p)+
     &              dnerd_dT(2*i-1,2*(j-1),p)+
     &              dnerd_dT(2*i-1,2*j-1,p)
            enddo               ! j
         enddo                  ! i
         do i=2,m-1
            dner_dT(i,m,p)=dnerd_dT(2*(i-1),md,p)+dnerd_dT(2*i-1,md,p)
         enddo

c     ner(ground-space)
         dner_dT(0,m+1,p)=dnerd_dT(0,md+1,p)

c     -- end of recombination
         
c     Symetrization
         do i=1,m
            do j=1,i-1
               dner_dT(i,j,p)=-dner_dT(j,i,p)
            enddo
            dner_dT(i,i,p)=0.0D+0
            dner_dT(i,0,p)=-dner_dT(0,i,p)
            dner_dT(m+1,i,p)=-dner_dT(i,m+1,p)
         enddo
         dner_dT(0,0,p)=0.0D+0
         dner_dT(m+1,m+1,p)=0.0D+0
         dner_dT(m+1,0,p)=-dner_dT(0,m+1,p)    
         
      enddo                     ! p

 123  continue
      return
      end
c     ...sensitivities




c     sensitivities...
      double precision function func_dB0_dTp(p,lambda)
      include 'max.inc'
      include 'data.inc'
      integer p
      double precision dTsol_dTp
      double precision dBdT,lambda
      
      if (p.eq.0) then
         dTsol_dTp=1.0D+0
      else
         dTsol_dTp=0.0D+0
      endif

      func_dB0_dTp=dBdT(Tsol,lambda)*dTsol_dTp

      return
      end
c     ...sensitivities




c     sensitivities...
      double precision function func_dBesp_dTp(p,lambda)
      include 'max.inc'
      include 'data.inc'
      integer p
      double precision dTesp_dTp
      double precision dBdT,lambda
      
      if (p.eq.m+1) then
         dTesp_dTp=1.0D+0
      else
         dTesp_dTp=0.0D+0
      endif

      func_dBesp_dTp=dBdT(Tesp,lambda)*dTesp_dTp

      return
      end
c     ...sensitivities





c     sensitivities...
      double precision function func_dB1_dTp(i,p,lambda)
      implicit none
      include 'max.inc'
      include 'data.inc'
      integer i,p,it
      double precision z1,Tz1,dTz1_dTp
      double precision dBdT,lambda

      it=i/2+1
      z1=alti(i-1)
      call find_T_gaz(z1,Tz1)

      if (i.eq.1) then
         if (p.eq.1) then
            dTz1_dTp=4.0D+0/3.0D+0
         else if (p.eq.2) then
            dTz1_dTp=-1.0D+0/3.0D+0
         else
            dTz1_dTp=0.0D+0
         endif
      else if (i.eq.2*m-2) then
         if (p.eq.m-1) then
            dTz1_dTp=1.0d+0-
     &           (e(m-1)-alt(m-2))/
     &           ((e(m)+e(m-1))/2.0D+0-alt(m-2))
         else if (p.eq.m) then
            dTz1_dTp=(e(m-1)-alt(m-2))/
     &           ((e(m)+e(m-1))/2.0D+0-alt(m-2))
         else
            dTz1_dTp=0.0D+0
         endif
      else
         
         if (dble(i)/2.0D+0.eq.dble(i/2)) then ! for 'i' even
            if (p.eq.it-1) then
               if (i.eq.2) then
                  dTz1_dTp=2.0D+0/3.0D+0
               else
                  dTz1_dTp=1.0D+0-
     &                 (e(it-1)-alt(it-2))/
     &                 (alt(it-1)-alt(it-2))
               endif
            else if (p.eq.it) then
               if (i.eq.2) then
                  dTz1_dTp=1.0D+0/3.0D+0
               else
                  dTz1_dTp=(e(it-1)-alt(it-2))/
     &                 (alt(it-1)-alt(it-2))
               endif
            else
               dTz1_dTp=0.0D+0
            endif
         else                   ! for 'i' odd
            if (p.eq.it) then
               dTz1_dTp=1.0D+0
            else
               dTz1_dTp=0.0D+0
            endif
         endif

      endif

      func_dB1_dTp=dBdT(Tz1,lambda)*dTz1_dTp

      return
      end
c     ...sensitivities





c     sensitivities...
      double precision function func_dB2_dTp(i,p,lambda)
      implicit none
      include 'max.inc'
      include 'data.inc'
      integer i,p,it
      double precision z2,Tz2,dTz2_dTp
      double precision dBdT,lambda

      it=i/2+1
      z2=alti(i)
      call find_T_gaz(z2,Tz2)

      if (i.eq.1) then
         if (p.eq.1) then
            dTz2_dTp=2.0D+0/3.0D+0
         else if (p.eq.2) then
            dTz2_dTp=1.0D+0/3.0D+0
         else
            dTz2_dTp=0.0D+0
         endif
      else if (i.eq.2*m-2) then
         if (p.eq.m-1) then
            dTz2_dTp=(alt(m-1)-alt(m-2))/
     &           ((e(m)+e(m-1))/2.0D+0-alt(m-2))
         else if (p.eq.m) then
            dTz2_dTp=-(alt(m-1)-(e(m)+e(m-1))/2.0D+0)/
     &           ((e(m)+e(m-1))/2.0D+0-alt(m-2))
         else
            dTz2_dTp=0.0D+0
         endif
      else
         
         if (dble(i)/2.0D+0.eq.dble(i/2)) then ! for 'i' even
            if (p.eq.it) then
               dTz2_dTp=1.0D+0
            else
               dTz2_dTp=0.0D+0
            endif
         else                   ! for 'i' odd
            if (p.eq.it) then
               if (it.eq.m-1) then
                  dTz2_dTp=1.0D+0-
     &                 (e(m-1)-alt(m-2))/
     &                 ((e(m)+e(m-1))/2.0D+0-alt(m-2))
               else
                  dTz2_dTp=1.0D+0-
     &                 (e(it)-alt(it-1))/
     &                 (alt(it)-alt(it-1))
               endif
            else if (p.eq.it+1) then
               if (it.eq.m-1) then
                  dTz2_dTp=(e(m-1)-alt(m-2))/
     &                 ((e(m)+e(m-1))/2.0D+0-alt(m-2))
               else
                  dTz2_dTp=(e(it)-alt(it-1))/
     &                 (alt(it)-alt(it-1))
               endif
            else
               dTz2_dTp=0.0D+0
            endif
         endif

      endif

      func_dB2_dTp=dBdT(Tz2,lambda)*dTz2_dTp

      return
      end
c     ...sensitivities





c     sensitivities...
      double precision function func_dtaui_dTp(i,p,dkadTp,band,quad)
      implicit none
      include 'max.inc'
      include 'data.inc'
      integer i,p,it,band,quad
      double precision z1,z2
      double precision dkadTp(1:Nbmx,1:Nqmx,1:Nmax)
      
      it=i/2+1
      if (p.eq.it) then
         z1=alti(i-1)
         z2=alti(i)
         func_dtaui_dTp=dkadTp(band,quad,it)*(z2-z1)
      else
         func_dtaui_dTp=0.0D+0
      endif

      return
      end
c     ...sensitivities



      
c     sensitivities...
      double precision function func_dtauB_dTp(i,j,p,dkadTp,band,quad)
      implicit none
      include 'max.inc'
      include 'data.inc'
      integer i,j,p,it,jt,band,quad,md
      double precision dkadTp(1:Nbmx,1:Nqmx,1:Nmax)
      double precision deltaz

      it=i/2+1
      jt=j/2+1
      md=2*m-2
      if (j.le.i+1) then
         func_dtauB_dTp=0.0D+0
         goto 123
      endif

      if ((i.eq.0).and.(j.gt.1)) then
         if (p.eq.0) then
            deltaz=0.0D+0
         else if ((p.gt.0).and.(p.lt.jt)) then
            deltaz=e(p)-e(p-1)
         else if (p.eq.jt) then
            if (dble(j)/2.0D+0.eq.dble(j/2)) then ! for 'j' even
               deltaz=0.0D+0
            else                ! for 'j' odd
               deltaz=(alt(jt-1)-e(jt-1))
            endif 
         else
            deltaz=0.0D+0
         endif
      else if ((i.ge.1).and.(j.eq.md+1)) then
         if (p.lt.it) then
            deltaz=0.0D+0
         else if (p.eq.it) then
            if (dble(i)/2.0D+0.eq.dble(i/2)) then ! for 'i' even
               deltaz=(e(it)-alt(it-1))
            else                ! for 'i' odd
               deltaz=0.0D+0
            endif 
         else if (p.gt.it) then
            deltaz=e(p)-e(p-1)
         endif
      else if ((i.gt.0).and.(i.lt.md+1).and.(j.gt.0).and.(j.lt.md+1))
     &        then
         if (p.lt.it) then
            deltaz=0.0D+0
         else if (p.eq.it) then
            if (dble(i)/2.0D+0.eq.dble(i/2)) then ! for 'i' even
               deltaz=(e(it)-alt(it-1))
            else                ! for 'i' odd
               deltaz=0.0D+0
            endif
         else if ((p.gt.it).and.(p.lt.jt)) then
            deltaz=e(p)-e(p-1)
         else if (p.eq.jt) then
            if (dble(j)/2.0D+0.eq.dble(j/2)) then ! for 'j' even
               deltaz=0.0D+0
            else                ! for 'i' odd
               deltaz=(alt(jt-1)-e(jt-1))
            endif
         else if (p.gt.jt) then
            deltaz=0.0D+0
         endif
      else
         write(*,*) 'Could not compute dtauB_dTp'
         write(*,*) 'i=',i,' j=',j,' p=',p
         stop
      endif

      func_dtauB_dTp=deltaz*dkadTp(band,quad,p)
 123  continue
      return
      end
c     ...sensitivities



c     sensitivities...
      double precision function func_dTi_dTp(it,p)
      implicit none
      include 'max.inc'
      include 'data.inc'
      integer it,p
      double precision dTi_dTp

      if (p.eq.it) then
         dTi_dTp=1.0D+0
      else
         dTi_dTp=0.0D+0
      endif
      func_dTi_dTp=dTi_dTp

      return
      end
c     ...sensitivities




c     sensitivities...
      subroutine dNERij_dTp_direct(B1,B2,B3,B4,
     &     dB1_dTp,dB2_dTp,dB3_dTp,dB4_dTp,
     &     tau_B,taui,tauj,
     &     dtaui_dTp,dtauj_dTp,dtauB_dTp,
     &     dNERij_dTp_dir)
      implicit none
      include 'param.inc'
c     Sensitivity of the direct part of NER(i,j) to a temperature.

      double precision expint
      double precision epsilon
      double precision B1,B2,B3,B4
      double precision dB1_dTp,dB2_dTp,dB3_dTp,dB4_dTp
      double precision tau_inf,tau_B,taui,tauj
      double precision tau_B1,tau_B2
      double precision dtaui_dTp,dtauj_dTp
      double precision dtauB1_dTp,dtauB2_dTp
      double precision dtau_inf_dTp,dtauB_dTp
      double precision alpha,beta
      double precision dNERij_dTp_dir

      alpha=expint(4,tau_B+taui)+expint(4,tau_B+tauj)
     &     -expint(4,tau_B)-expint(4,tau_B+taui+tauj)

      beta=(dtauB_dTp+dtaui_dTp)*expint(3,tau_B+taui)
     &     +(dtauB_dTp+dtauj_dTp)*expint(3,tau_B+tauj)
     &     -dtauB_dTp*expint(3,tau_B)
     &     -(dtauB_dTp+dtaui_dTp+dtauj_dTp)*
     &     expint(3,tau_B+taui+tauj)

      dNERij_dTp_dir=-2.0D+0*pi*(
     &     (dB2_dTp-dB3_dTp)*expint(3,tau_B)-
     &     (B2-B3)*dtauB_dTp*expint(2,tau_B)-
     &     (dB1_dTp-dB3_dTp)*expint(3,tau_B+taui)+
     &     (B1-B3)*(dtauB_dTp+dtaui_dTp)*
     &     expint(2,tau_B+taui)-
     &     (dB2_dTp-dB4_dTp)*expint(3,tau_B+tauj)+
     &     (B2-B4)*(dtauB_dTp+dtauj_dTp)*
     &     expint(2,tau_B+tauj)+
     &     (dB1_dTp-dB4_dTp)*expint(3,tau_B+taui+tauj)-
     &     (B1-B4)*(dtauB_dTp+dtaui_dTp+dtauj_dTp)*
     &     expint(2,tau_B+taui+tauj)
     &     +1.0D+0/taui**2*(
     &     taui*((dB2_dTp-dB1_dTp)*alpha-
     &     (B2-B1)*beta)
     &     -dtaui_dTp*(B2-B1)*alpha)
     &     +1.0D+0/tauj**2*(
     &     tauj*((dB4_dTp-dB3_dTp)*alpha-
     &     (B4-B3)*beta)
     &     -dtauj_dTp*(B4-B3)*alpha))

      return
      end
c     ...sensitivities




c     sensitivities...
      subroutine dNERij_dTp_specular_reflection_virtual(epsilon,
     &     B1,B2,B3,B4,
     &     dB1_dTp,dB2_dTp,dB3_dTp,dB4_dTp,
     &     tau_inf,tau_B,taui,tauj,
     &     dtaui_dTp,dtauj_dTp,
     &     dtau_inf_dTp,dtauB_dTp,
     &     dNERij_dTp_ref)
      implicit none
      include 'param.inc'
c     Sensitivity of the NER between layers (i) and (j) via specular ground reflection
c     to a temperature. This routine will use a virtual layer (-i) that is the symetrical
c     of layer (i) with respect to the ground.

      double precision expint
      double precision epsilon
      double precision B1,B2,B3,B4
      double precision dB1_dTp,dB2_dTp,dB3_dTp,dB4_dTp
      double precision tau_inf,tau_B,taui,tauj
      double precision tau_B1,tau_B2
      double precision dtaui_dTp,dtauj_dTp
      double precision dtauB1_dTp,dtauB2_dTp
      double precision dtau_inf_dTp,dtauB_dTp
      double precision B1_ind,B2_ind,B3_ind,B4_ind
      double precision dB1_dTp_ind,dB2_dTp_ind,dB3_dTp_ind,dB4_dTp_ind
      double precision taui_ind,tauj_ind,tau_B_ind
      double precision dtaui_dTp_ind,dtauj_dTp_ind,dtauB_dTp_ind
      double precision alpha_ind,beta_ind
      double precision dNERmij_dTp_dir,dNERij_dTp_ref

      B1_ind=B2
      B2_ind=B1
      B3_ind=B3
      B4_ind=B4
      dB1_dTp_ind=dB2_dTp
      dB2_dTp_ind=dB1_dTp
      dB3_dTp_ind=dB3_dTp
      dB4_dTp_ind=dB4_dTp
      taui_ind=taui
      tauj_ind=tauj
      tau_B_ind=2.0D+0*tau_inf+taui+tau_B
      dtaui_dTp_ind=dtaui_dTp
      dtauj_dTp_ind=dtauj_dTp
      dtauB_dTp_ind=2.0D+0*dtau_inf_dTp+dtaui_dTp+dtauB_dTp

      call dNERij_dTp_direct(B1_ind,B2_ind,B3_ind,B4_ind,
     &     dB1_dTp_ind,dB2_dTp_ind,dB3_dTp_ind,dB4_dTp_ind,
     &     tau_B_ind,taui_ind,tauj_ind,
     &     dtaui_dTp_ind,dtauj_dTp_ind,dtauB_dTp_ind,
     &     dNERmij_dTp_dir) ! dNERmij_dTp_dir=W/m/m/K
      dNERij_dTp_ref=(1.0D+0-epsilon)*dNERmij_dTp_dir
      
      return
      end
c     ...sensitivities



c     sensitivities...
      subroutine dNERij_dTp_specular_reflection_real(epsilon,
     &     B1,B2,B3,B4,
     &     dB1_dTp,dB2_dTp,dB3_dTp,dB4_dTp,
     &     tau_inf,tau_B,taui,tauj,
     &     dtaui_dTp,dtauj_dTp,
     &     dtau_inf_dTp,dtauB_dTp,
     &     dNERij_dTp_ref)
      implicit none
      include 'param.inc'
c     Sensitivity of the NER between layers (i) and (j) via specular ground reflection
c     to a temperature. This routine uses the derived expression of the reflected NER(i,j).
c     The result of the present routine is perfectly equivalent to the result
c     given by routine "dNERij_dTp_specular_reflection_virtual"

      double precision expint
      double precision epsilon
      double precision B1,B2,B3,B4
      double precision dB1_dTp,dB2_dTp,dB3_dTp,dB4_dTp
      double precision tau_inf,tau_B,taui,tauj
      double precision tau_B1,tau_B2
      double precision dtaui_dTp,dtauj_dTp
      double precision dtauB1_dTp,dtauB2_dTp
      double precision dtau_inf_dTp,dtauB_dTp
      double precision a(1:6)
      double precision da_dTp(1:6)
      double precision dNERij_dTp_ref

      tau_B1=tau_inf
      tau_B2=tau_B1+taui+tau_B
      dtauB1_dTp=dtau_inf_dTp
      dtauB2_dTp=dtauB1_dTp+dtaui_dTp+dtauB_dTp
      call a_coeffs(B1,B2,B3,B4,taui,tauj,a)
      call da_dTp_coeffs(B1,B2,B3,B4,taui,tauj,
     &     dB1_dTp,dB2_dTp,dB3_dTp,dB4_dTp,
     &     dtaui_dTp,dtauj_dTp,da_dTp)

      dNERij_dTp_ref=2.0D+0*pi*(1.0D+0-epsilon)*
     &     (
     &     da_dTp(1)*expint(3,tau_B1+tau_B2)
     &     -a(1)*(dtauB1_dTp+dtauB2_dTp)*expint(2,tau_B1+tau_B2)
     &     +(da_dTp(2)+da_dTp(4))*expint(4,tau_B1+tau_B2)
     &     -(a(2)+a(4))*(dtauB1_dTp+dtauB2_dTp)*expint(3,tau_B1+tau_B2)
     &     +da_dTp(3)*expint(3,tau_B1+tau_B2+taui)
     &     -a(3)*(dtauB1_dTp+dtauB2_dTp+dtaui_dTp)
     &     *expint(2,tau_B1+tau_B2+taui)
     &     -(da_dTp(2)+da_dTp(4))*expint(4,tau_B1+tau_B2+taui)
     &     +(a(2)+a(4))*(dtauB1_dTp+dtauB2_dTp+dtaui_dTp)
     &     *expint(3,tau_B1+tau_B2+taui)
     &     +da_dTp(5)*expint(3,tau_B1+tau_B2+tauj)
     &     -a(5)*(dtauB1_dTp+dtauB2_dTp+dtauj_dTp)
     &     *expint(2,tau_B1+tau_B2+tauj)
     &     -(da_dTp(2)+da_dTp(4))*expint(4,tau_B1+tau_B2+tauj)
     &     +(a(2)+a(4))*(dtauB1_dTp+dtauB2_dTp+dtauj_dTp)
     &     *expint(3,tau_B1+tau_B2+tauj)
     &     +da_dTp(6)*expint(3,tau_B1+tau_B2+taui+tauj)
     &     -a(6)*(dtauB1_dTp+dtauB2_dTp+dtaui_dTp+dtauj_dTp)
     &     *expint(2,tau_B1+tau_B2+taui+tauj)
     &     +(da_dTp(2)+da_dTp(4))*expint(4,tau_B1+tau_B2+taui+tauj)
     &     -(a(2)+a(4))*(dtauB1_dTp+dtauB2_dTp+dtaui_dTp+dtauj_dTp)
     &     *expint(3,tau_B1+tau_B2+taui+tauj)
     &     )                    ! dNERij_dTp_ref=W/m/m/K

      return
      end
c     ...sensitivities



c     sensitivities...
      subroutine dNERij_dTp_diffuse_reflection(epsilon,
     &     B1,B2,B3,B4,
     &     dB1_dTp,dB2_dTp,dB3_dTp,dB4_dTp,
     &     tau_inf,tau_B,taui,tauj,
     &     dtaui_dTp,dtauj_dTp,
     &     dtau_inf_dTp,dtauB_dTp,
     &     dNERij_dTp_ref)
      implicit none
      include 'param.inc'
c     Sensitivity of the NER between layers (i) and (j) via diffuse ground reflection
c     to a temperature.

      double precision expint
      double precision epsilon
      double precision B1,B2,B3,B4
      double precision dB1_dTp,dB2_dTp,dB3_dTp,dB4_dTp
      double precision tau_inf,tau_B,taui,tauj
      double precision tau_B1,tau_B2
      double precision dtaui_dTp,dtauj_dTp
      double precision dtauB1_dTp,dtauB2_dTp
      double precision dtau_inf_dTp,dtauB_dTp
      double precision alpha3,beta3,gamma3,delta3
      double precision dalpha3_dTp,dbeta3_dTp,dgamma3_dTp,ddelta3_dTp
      double precision dNERij_dTp_ref

      tau_B1=tau_inf
      tau_B2=tau_B1+taui+tau_B
      dtauB1_dTp=dtau_inf_dTp
      dtauB2_dTp=dtauB1_dTp+dtaui_dTp+dtauB_dTp
      call coeffs3(B1,B2,B3,B4,taui,tauj,tau_B1,tau_B2,
     &     alpha3,beta3,gamma3,delta3)

      call dcoeffs3_dTp(B1,B2,B3,B4,taui,tauj,tau_B1,tau_B2,
     &     dB1_dTp,dB2_dTp,dB3_dTp,dB4_dTp,
     &     dtaui_dTp,dtauj_dTp,dtauB1_dTp,dtauB2_dTp,
     &     dalpha3_dTp,dbeta3_dTp,dgamma3_dTp,ddelta3_dTp)

      dNERij_dTp_ref=4.0D+00*pi*(1.0D+0-epsilon)*
     &     (
     &     dalpha3_dTp*expint(3,tau_B1)
     &     -alpha3*dtauB1_dTp*expint(2,tau_B1)
     &     +dbeta3_dTp*expint(4,tau_B1)
     &     -beta3*dtauB1_dTp*expint(3,tau_B1)
     &     +dgamma3_dTp*expint(3,tau_B1+taui)
     &     -gamma3*(dtauB1_dTp+dtaui_dTp)*expint(2,tau_B1+taui)
     &     +ddelta3_dTp*expint(4,tau_B1+taui)
     &     -delta3*(dtauB1_dTp+dtaui_dTp)*expint(3,tau_B1+taui)
     &     )                    ! dNERij_dTp_ref=W/m/m/K

      return
      end
c     ...sensitivities



c     sensitivities...
      subroutine da_dTp_coeffs(B1,B2,B3,B4,
     &     taui,tauj,
     &     dB1_dTp,dB2_dTp,dB3_dTp,dB4_dTp,
     &     dtaui_dTp,dtauj_dTp,
     &     da_dTp)
      implicit none
      double precision B1,B2,B3,B4
      double precision taui,tauj
      double precision dB1_dTp,dB2_dTp,dB3_dTp,dB4_dTp
      double precision dtaui_dTp,dtauj_dTp
      double precision da_dTp(1:6)

      da_dTp(1)=dB3_dTp-dB1_dTp
      da_dTp(2)=(dB1_dTp-dB2_dTp)/taui+dtaui_dTp*(B2-B1)/taui**2
      da_dTp(3)=dB2_dTp-dB3_dTp
      da_dTp(4)=(dB4_dTp-dB3_dTp)/tauj+dtauj_dTp*(B3-B4)/tauj**2
      da_dTp(5)=dB1_dTp-dB4_dTp
      da_dTp(6)=dB4_dTp-dB2_dTp

      return
      end
c     ...sensitivities


c     sensitivities...
      subroutine dcoeffs3_dTp(B1,B2,B3,B4,taui,tauj,tau_B1,tau_B2,
     &     dB1_dTp,dB2_dTp,dB3_dTp,dB4_dTp,
     &     dtaui_dTp,dtauj_dTp,dtauB1_dTp,dtauB2_dTp,
     &     dalpha3_dTp,dbeta3_dTp,dgamma3_dTp,ddelta3_dTp)
      implicit none
      double precision expint
      double precision B1,B2,B3,B4,taui,tauj,tau_B1,tau_B2
      double precision dB1_dTp,dB2_dTp,dB3_dTp,dB4_dTp
      double precision dtaui_dTp,dtauj_dTp
      double precision dtauB1_dTp,dtauB2_dTp
      double precision a(1:6)
      double precision da_dTp(1:6)
      double precision dalpha3_dTp,dbeta3_dTp,dgamma3_dTp,ddelta3_dTp

      call a_coeffs(B1,B2,B3,B4,taui,tauj,a)

      call da_dTp_coeffs(B1,B2,B3,B4,taui,tauj,
     &     dB1_dTp,dB2_dTp,dB3_dTp,dB4_dTp,
     &     dtaui_dTp,dtauj_dTp,da_dTp)

      dalpha3_dTp=da_dTp(1)*expint(3,tau_B2)
     &     -a(1)*dtauB2_dTp*expint(2,tau_B2)
     &     +da_dTp(4)*expint(4,tau_B2)
     &     -a(4)*dtauB2_dTp*expint(3,tau_B2)
     &     +da_dTp(5)*expint(3,tau_B2+tauj)
     &     -a(5)*(dtauB2_dTp+dtauj_dTp)*expint(2,tau_B2+tauj)
     &     -da_dTp(4)*expint(4,tau_B2+tauj)
     &     +a(4)*(dtauB2_dTp+dtauj_dTp)*expint(3,tau_B2+tauj)

      dbeta3_dTp=da_dTp(2)*(expint(3,tau_B2)-expint(3,tau_B2+tauj))
     &     -a(2)*(dtauB2_dTp*expint(2,tau_B2)-
     &     (dtauB2_dTp+dtauj_dTp)*expint(2,tau_B2+tauj))

      dgamma3_dTp=da_dTp(3)*expint(3,tau_B2)
     &     -a(3)*dtauB2_dTp*expint(2,tau_B2)
     &     -da_dTp(4)*expint(4,tau_B2)
     &     +a(4)*dtauB2_dTp*expint(3,tau_B2)
     &     +da_dTp(6)*expint(3,tau_B2+tauj)
     &     -a(6)*(dtauB2_dTp+dtauj_dTp)*expint(2,tau_B2+tauj)
     &     +da_dTp(4)*expint(4,tau_B2+tauj)
     &     -a(4)*(dtauB2_dTp+dtauj_dTp)*expint(3,tau_B2+tauj)

      ddelta3_dTp=-dbeta3_dTp

      return
      end
c     ...sensitivities
