c     Copyright (C)
c     2000 -2013 Universite Paul Sabatier,
c     2000 -2013 Centre National de la Recherche Scientifique
c     2000 -2013 Ecole des Mines d'Albi
c     2014-2018 Msostar
c
c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c
      subroutine fusion(PNE,var_PNE,fasc,var_fasc,fdes,var_fdes,
     &     PNErec,var_PNErec,fascrec,var_fascrec,
     &     fdesrec,var_fdesrec)
      implicit none

c     Cette routine permet de recombiner les mailles de calcul en mailles
c     de rsultat.
c
c     Entres :
c       + PNE : matrice (non symtrique) des PNE
c       + var_PNE : matrice des variances sur chaque PNE
c
c     Sorties :
c       + PNErec : matrice (non symtrique) des PNE recombines
c       + var_PNErec : matrice des variances de PNE recombines
c

      include 'max.inc'
      include 'param.inc'
      include 'data.inc'
      integer i,j,k
      double precision PNE(0:Nmax,0:Nmax),var_PNE(0:Nmax,0:Nmax)
      double precision PNErec(0:Nmax,0:Nmax),var_PNErec(0:Nmax,0:Nmax)
      double precision fasc(0:Nmax,0:Nmax),fdes(0:Nmax,0:Nmax)
      double precision var_fasc(0:Nmax,0:Nmax),var_fdes(0:Nmax,0:Nmax)
      double precision fascrec(0:Nmax,0:Nmax),fdesrec(0:Nmax,0:Nmax)
      double precision var_fascrec(0:Nmax,0:Nmax)
      double precision var_fdesrec(0:Nmax,0:Nmax)


c     Test
c      m=4
c      n=3
c     Test

      do i=2,n-1
         do j=2,n-1
            PNErec(i,j)=PNE(2*i-2,2*j-2)+PNE(2*i-2,2*j-1)
     &                 +PNE(2*i-1,2*j-2)+PNE(2*i-1,2*j-1)
            var_PNErec(i,j)=var_PNE(2*i-2,2*j-2)+var_PNE(2*i-2,2*j-1)
     &                     +var_PNE(2*i-1,2*j-2)+var_PNE(2*i-1,2*j-1)
            if (i.eq.j) then
               PNErec(i,j)=0.0D+0
               var_PNErec(i,j)=0.0D+0
            endif
         enddo
      enddo
      PNErec(1,1)=0.0D+0
      PNErec(1,n)=PNE(1,2*n-2)
      PNErec(n,1)=PNE(2*n-2,1)
      PNErec(n,n)=0.0D+0
      PNErec(0,0)=0.0D+0
      PNErec(n+1,n+1)=0.0D+0
      PNErec(0,n+1)=PNE(0,2*n-1)
      PNErec(n+1,0)=PNE(2*n-1,0)
      PNErec(0,1)=PNE(0,1)
      PNErec(0,n)=PNE(0,n)
      PNErec(n+1,1)=PNE(2*n-1,1)
      PNErec(n+1,n)=PNE(2*n-1,2*n-2)
      PNErec(1,0)=PNE(1,0)
      PNErec(n,0)=PNE(2*n-2,0)
      PNErec(1,n+1)=PNE(1,2*n-1)
      PNErec(n,n+1)=PNE(2*n-2,2*n-1)

      var_PNErec(1,1)=0.0D+0
      var_PNErec(1,n)=var_PNE(1,2*n-2)
      var_PNErec(n,1)=var_PNE(2*n-2,1)
      var_PNErec(n,n)=0.0D+0
      var_PNErec(0,0)=0.0D+0
      var_PNErec(n+1,n+1)=0.0D+0
      var_PNErec(0,n+1)=var_PNE(0,2*n-1)
      var_PNErec(n+1,0)=var_PNE(2*n-1,0)
      var_PNErec(0,1)=var_PNE(0,1)
      var_PNErec(0,n)=var_PNE(0,n)
      var_PNErec(n+1,1)=var_PNE(2*n-1,1)
      var_PNErec(n+1,n)=var_PNE(2*n-1,2*n-2)
      var_PNErec(1,0)=var_PNE(1,0)
      var_PNErec(n,0)=var_PNE(2*n-2,0)
      var_PNErec(1,n+1)=var_PNE(1,2*n-1)
      var_PNErec(n,n+1)=var_PNE(2*n-2,2*n-1)

      do k=2,n-1
         PNErec(k,1)=PNE(2*k-2,1)+PNE(2*k-1,1)
         var_PNErec(k,1)=var_PNE(2*k-2,1)+var_PNE(2*k-1,1)
         PNErec(1,k)=PNE(1,2*k-2)+PNE(1,2*k-1)
         var_PNErec(1,k)=var_PNE(1,2*k-2)+var_PNE(1,2*k-1)
         PNErec(k,n)=PNE(2*k-2,n)+PNE(2*k-1,n)
         var_PNErec(k,n)=var_PNE(2*k-2,n)+var_PNE(2*k-1,n)
         PNErec(n,k)=PNE(n,2*k-2)+PNE(n,2*k-1)
         var_PNErec(n,k)=var_PNE(n,2*k-2)+var_PNE(n,2*k-1)
         PNErec(k,0)=PNE(2*k-2,0)+PNE(2*k-1,0)
         var_PNErec(k,0)=var_PNE(2*k-2,0)+var_PNE(2*k-1,0)
         PNErec(0,k)=PNE(0,2*k-2)+PNE(0,2*k-1)
         var_PNErec(0,k)=var_PNE(0,2*k-2)+var_PNE(0,2*k-1)
         PNErec(k,n+1)=PNE(2*k-2,2*n-1)+PNE(2*k-1,2*n-1)
         var_PNErec(k,n+1)=var_PNE(2*k-2,2*n-1)+var_PNE(2*k-1,2*n-1)
         PNErec(n+1,k)=PNE(2*n-1,2*k-2)+PNE(2*n-1,2*k-1)
         var_PNErec(n+1,k)=var_PNE(2*n-1,2*k-2)+var_PNE(2*n-1,2*k-1)
      enddo

      do j=0,1
         fascrec(j,0)=fasc(j,0)
         fascrec(j,1)=fasc(j,1)
         fascrec(j,n)=fasc(j,2*n-2)
         var_fascrec(j,0)=var_fasc(j,0)
         var_fascrec(j,1)=var_fasc(j,1)
         var_fascrec(j,n)=var_fasc(j,2*n-2)
         fdesrec(j,0)=fdes(j,0)
         fdesrec(j,1)=fdes(j,1)
         fdesrec(j,n)=fdes(j,2*n-2)
         var_fdesrec(j,0)=var_fdes(j,0)
         var_fdesrec(j,1)=var_fdes(j,1)
         var_fdesrec(j,n)=var_fdes(j,2*n-2)
         do i=2,n-1
            fascrec(j,i)=fasc(j,2*i-1)
            var_fascrec(j,i)=var_fasc(j,2*i-1)
            fdesrec(j,i)=fdes(j,2*i-1)
            var_fdesrec(j,i)=var_fdes(j,2*i-1)
         enddo
      enddo
      
      do i=2,n-1
         do j=2,n-1
            fascrec(i,j)=fasc(2*i-2,2*j-1)+fasc(2*i-1,2*j-1)
            var_fascrec(i,j)=var_fasc(2*i-2,2*j-1)+
     &           var_fasc(2*i-1,2*j-1)
            fdesrec(i,j)=fdes(2*i-2,2*j-1)+fdes(2*i-1,2*j-1)
            var_fdesrec(i,j)=var_fdes(2*i-2,2*j-1)+
     &           var_fdes(2*i-1,2*j-1)
         enddo
      enddo
      
      fascrec(n,0)=fasc(2*n-2,0)
      fascrec(n,1)=fasc(2*n-2,1)
      fascrec(n,n)=fasc(2*n-2,2*n-2)
      var_fascrec(n,0)=var_fasc(2*n-2,0)
      var_fascrec(n,1)=var_fasc(2*n-2,1)
      var_fascrec(n,n)=var_fasc(2*n-2,2*n-2)
      fdesrec(n,0)=fdes(2*n-2,0)
      fdesrec(n,1)=fdes(2*n-2,1)
      fdesrec(n,n)=fdes(2*n-2,2*n-2)
      var_fdesrec(n,0)=var_fdes(2*n-2,0)
      var_fdesrec(n,1)=var_fdes(2*n-2,1)
      var_fdesrec(n,n)=var_fdes(2*n-2,2*n-2)
      do i=2,n-1
         fascrec(n,i)=fasc(2*n-2,2*i-1)
         var_fascrec(n,i)=var_fasc(2*n-2,2*i-1)
         fdesrec(n,i)=fdes(2*n-2,2*i-1)
         var_fdesrec(n,i)=var_fdes(2*n-2,2*i-1)
      enddo
      fascrec(n+1,0)=fasc(2*n-1,0)
      fascrec(n+1,1)=fasc(2*n-1,1)
      fascrec(n+1,n)=fasc(2*n-1,2*n-2)
      var_fascrec(n+1,0)=var_fasc(2*n-1,0)
      var_fascrec(n+1,1)=var_fasc(2*n-1,1)
      var_fascrec(n+1,n)=var_fasc(2*n-1,2*n-2)
      fdesrec(n+1,0)=fdes(2*n-1,0)
      fdesrec(n+1,1)=fdes(2*n-1,1)
      fdesrec(n+1,n)=fdes(2*n-1,2*n-2)
      var_fdesrec(n+1,0)=var_fdes(2*n-1,0)
      var_fdesrec(n+1,1)=var_fdes(2*n-1,1)
      var_fdesrec(n+1,n)=var_fdes(2*n-1,2*n-2)
      do i=2,n-1
         fascrec(n+1,i)=fasc(2*n-1,2*i-1)
         var_fascrec(n+1,i)=var_fasc(2*n-1,2*i-1)
         fdesrec(n+1,i)=fdes(2*n-1,2*i-1)
         var_fdesrec(n+1,i)=var_fdes(2*n-1,2*i-1)
      enddo
      
      return
      end
