c     Copyright (C)
c     2000 -2013 Universite Paul Sabatier,
c     2000 -2013 Centre National de la Recherche Scientifique
c     2000 -2013 Ecole des Mines d'Albi
c     2014-2018 Msostar
c
c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c
      subroutine pdf_freq(md,sens,Nb,Nq,nb_pdfh,cumul_pdfh,nb_pdfb,
     &     cumul_pdfb,opth,optb,w,band,quad)
      implicit none
      include 'max.inc'
      include 'data.inc'
      include 'options.inc'
      include 'freq.inc'
      double precision w,R
      double precision cumul_w_moy(0:Nqmx)
      integer i,k,p,q,n_gaz,md,kp,sens,Nb,Nq,band,quad,out
      integer c,kl,kh,tc
      
c     Cette routine permet de choisir un gaz gris parmis les Nb*Nq gaz gris donns
c     et retourne le pdf associ au choix de ce gaz gris
c
c     Entres :
c       + md : numro de la maille d'mission [0..m+1]
c       + sens : sens=0 : tir vers le bas ; sens=1 : tir vers le haut
c       + Nb : nombre de bandes troites
c       + Nq : ordre de la quadrature
c       + nb_pdf : tableau des pdf de pondration des bandes troites pour chaque maille
c       + cumul_pdf : tableau des cumules des pdf de pondration des bandes troites
c
c     Sorties :
c       + w : poids associ aux choix de 'band' et 'quad'
c       + band : numro de la bande troite
c       + quad : numro de la quadrature
c
      
      tc=0
      n_gaz=Nb*Nq
      call rand_uniforme(R)
      
      c=int(R*Na)+1
      if (sens.eq.0) then
         kl=int(optb(md,c-1))
         kh=int(optb(md,c))
         do k=kl,kh
            if ((R.ge.cumul_pdfb(md,k-1)).and.
     &           (R.le.cumul_pdfb(md,k))) then
               kp=k
               call nbands(kp,Nq,band,quad)
               w=1.0D+0/nb_pdfb(md,kp)
               tc=1
               goto 321
            endif
         enddo
      else
         kl=int(opth(md,c-1))
         kh=int(opth(md,c))
         do k=kl,kh
            if ((R.ge.cumul_pdfh(md,k-1)).and.
     &           (R.le.cumul_pdfh(md,k))) then
               kp=k
               call nbands(kp,Nq,band,quad)
               w=1.0D+0/nb_pdfh(md,kp)
               tc=1
               goto 321
            endif
         enddo
      endif

 321  continue

c     Debug
      if ((band.lt.1).or.(band.gt.Nb)) then
         write(*,*) 'k=',k,' band=',band
         if (sens.eq.0) then
            write(*,*) R,cumul_pdfb(md,k-1),cumul_pdfb(md,k)
         else if (sens.eq.1) then
            write(*,*) R,cumul_pdfh(md,k-1),cumul_pdfh(md,k)
         endif
         write(*,*) 'computation will abort now'
         stop
      endif
c     Debug

      if (tc.eq.0) then
         call screen(1112,c-1,c,0.0D+0,0.0D+0)
         if (sens.eq.0) then
            do c=0,Na
               call screen(1113,md,c,optb(md,c),0.0D+0)
            enddo
         else
            do c=0,Na
               call screen(1114,md,c,opth(md,c),0.0D+0)
            enddo
         endif
         call screen(1115,kl,kh,R,0.0D+0)
         stop
      endif

      return
      end
