c     Copyright (C)
c     2000 -2013 Universite Paul Sabatier,
c     2000 -2013 Centre National de la Recherche Scientifique
c     2000 -2013 Ecole des Mines d'Albi
c     2014-2018 Msostar
c
c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c
      subroutine calc_tau_abs_mlay(band,quad,i_start,i_end,tab,
     &     tau_abs)
      implicit none
      include 'max.inc'
      include 'param.inc'
      include 'data.inc'
      double precision tau_abs,tau_int,ray(0:DiffMax,2)
      integer i,i_start,i_end,tab,band,quad
c     Calcule l'paisseur optique d'absorption des mailles i_start
c      i_end (bornes comprises)
c     
c     tab=1 => cette routine est appele par "init_nb_pdf" et le tableau des altitudes
c               utiliser est "alti"
c     tab=2 => cette routine est appele par "wall" et le tableau des altitudes
c               utiliser est "e"
      
      ray(0,1)=0.0D+0
      ray(0,2)=0.0D+0

      tau_abs=0.0D+0
      if (i_start.le.i_end) then
         do i=i_start,i_end
            ray(1,2)=0.0D+0
            if (tab.eq.1) then
               ray(1,1)=alti(i-1)
               ray(2,1)=alti(i)
            else if (tab.eq.2) then
               ray(1,1)=e(i-1)
               ray(2,1)=e(i)
            endif
            ray(2,2)=ray(2,1)-ray(1,1)
            if (tab.eq.1) then
               call find_tau_abs(band,quad,int(i/2)+1,ray,2,tau_int)
            else if (tab.eq.2) then
               call find_tau_abs(band,quad,i,ray,2,tau_int)
            endif
            tau_abs=tau_abs+tau_int
         enddo
      endif

      return
      end



      subroutine calc_tau_diff_mlay(band,quad,i_start,i_end,tab,
     &     tau_diff)
      implicit none
      include 'max.inc'
      include 'param.inc'
      include 'data.inc'
      double precision tau_diff,tau_int,ray(0:DiffMax,2)
      integer i,i_start,i_end,tab,band,quad
c     Calcule l'paisseur optique de diffusion des mailles i_start
c      i_end (bornes comprises)
c     
c     tab=1 => cette routine est appele par "init_nb_pdf" et le tableau des altitudes
c               utiliser est "alti"
c     tab=2 => cette routine est appele par "wall" et le tableau des altitudes
c               utiliser est "e"
      
      ray(0,1)=0.0D+0
      ray(0,2)=0.0D+0
      
      tau_diff=0.0D+0
      if (i_start.lt.i_end) then
         do i=i_start,i_end
            ray(1,2)=0.0D+0
            if (tab.eq.1) then
               ray(1,1)=alti(i-1)
               ray(2,1)=alti(i)
            else if (tab.eq.2) then
               ray(1,1)=e(i-1)
               ray(2,1)=e(i)
            endif
            ray(2,2)=ray(2,1)-ray(1,1)
            if (tab.eq.1) then
            call find_tau_diff(band,quad,int(i/2)+1,ray,2,tau_int)
         else if (tab.eq.2) then
            call find_tau_diff(band,quad,i,ray,2,tau_int)
         endif
            tau_diff=tau_diff+tau_int
         enddo
      endif

      return
      end


      subroutine find_tau_abs(band,quad,i,ray,r_max,tau_abs)
      implicit none
      include 'max.inc'
      include 'param.inc'
      include 'data.inc'
      double precision ray(0:DiffMax,2),tau_abs,ka
      integer r_max,r,i,band,quad
c     Calcule l'paisseur optique d'absorption le long d'un chemin de diffusion
c     dcrit par le tableau 'ray' (ray doit appartenir  la maille i)
c
c     Entres :
c       + i : numro de la maille courante (dans tableau 'e')
c       + ray : tableau de chemin
c       + r_max : indice reprant la fin du chemin dans 'ray'
c
c     Sorties :
c       + tau_abs : paisseur optique d'absorption du chemin
c

      
      tau_abs=0.0D+0
      if (r_max.ge.2) then
         do r=2,r_max           ! Intgration de l'ep. opt. sur un segment de droite
            call find_ka(band,quad,i,(ray(r-1,1)+ray(r,1))/2.0D+0,ka)
            tau_abs=tau_abs+ka*(ray(r,2)-ray(r-1,2))
         enddo
      endif
c     Debug
c      if (tau_abs.lt.0.0D+0) then
c         write(*,*) 'tau_abs:',tau_abs
c         write(*,*) 'ka:',ka
c         write(*,*) 'ray(',r,',2):',ray(r,2),' ray(',r-1,',2):',
c     &        ray(r-1,2)
c      endif
c     Debug
      return
      end


      subroutine find_tau_diff(band,quad,i,ray,r_max,tau_diff)
      implicit none
      include 'max.inc'
      include 'param.inc'
      include 'data.inc'
      double precision ray(0:DiffMax,2),tau_diff,kd
      integer r_max,r,i,band,quad
c     Calcule l'paisseur optique de diffusion le long d'un chemin de diffusion
c     dcrit par le tableau 'ray', dans la maille i
c
c     Entres :
c       + i : indice de la maille de calcul
c       + ray : tableau de chemin
c       + r_max : indice reprant la fin du chemin dans 'ray'
c
c     Sorties :
c       + tau_diff : paisseur optique de diffusion du chemin
c
     
      tau_diff=0.0D+0
      if (r_max.ge.2) then
         do r=2,r_max           ! Intgration de l'ep. opt. sur un segment de droite
            call find_kd(band,quad,i,(ray(r-1,1)+ray(r,1))/2.0D+0,kd)
            tau_diff=tau_diff+kd*(ray(r,2)-ray(r-1,2))
         enddo
      endif
      return
      end


      subroutine find_z1_abs(band,quad,j,z0,alpha,tau_abs,z1)
      implicit none
      include 'max.inc'
      include 'data.inc'
      double precision z0,alpha,tau_abs,z1,ka,z
      integer j,band,quad
c     Calcule l'altitude z1 atteinte  partir de l'altitude z0 en parcourant
c     une paisseur optique d'absorption tau_abs suivant la direction alpha
c     Entres :
c       + j : numro de la maille courante o se calcule le trajet du rayon
c       + z0 : altitude de dpart du segment
c       + alpha : angle de parcours (par rapport  la verticale descendante)
c       + tau_abs : paisseur optique d'absorption du segment
c     
c     Sorties :
c       + z1 : altitude de fin du segment
c
      z=(e(j)+e(j-1))/2.0D+0
c     On prend le ka du centre de maille pour tre certain de ne pas
c     prendre le ka de l'interface (entraine des incohrences)
      call find_ka(band,quad,j,z,ka)
      z1=z0-dcos(alpha)*tau_abs/ka
c     Debug
      if (z1.lt.0.0D+0) then
         write(*,*) 'z1=',z1
         write(*,*) 'alpha=',alpha
         write(*,*) 'z0=',z0
         write(*,*) 'tau_abs=',tau_abs
         write(*,*) 'ka=',ka
         write(*,*) 'band=',band
         write(*,*) 
      endif
c     Debug
      return
      end


      subroutine find_z1_diff(band,quad,j,z0,alpha,tau_diff,z1)
      implicit none
      include 'max.inc'
      include 'data.inc'
      double precision z0,alpha,tau_diff,z1,kd,z
      integer j,band,quad
c     Calcule l'altitude z1 atteinte  partir de l'altitude z0 en parcourant
c     une paisseur optique de diffusion tau_diff suivant la direction alpha
c     Entres :
c       + j : numro de la maille courante o se calcule le trajet du rayon
c       + z0 : altitude de dpart du segment
c       + alpha : angle de parcours (par rapport  la verticale descendante)
c       + tau_diff : paisseur optique de diffusion du segment
c     
c     Sorties :
c       + z1 : altitude de fin du segment
c
      z=(e(j)+e(j-1))/2.0D+0
      call find_kd(band,quad,j,z,kd)
      z1=z0-dcos(alpha)*tau_diff/kd
c     Debug
c      write(*,*) 'z:',z,' j:',j
c      write(*,*) 'kd:',kd,' z1:',z1
c     Debug
      return
      end
